#ifndef _FS_H
#define _FS_H

#include <fuse/fuse.h>
#include <fuse/fuse_opt.h>
#include <errno.h>
#include <libgen.h>
#include <assert.h>

#include <inode.h>
#include <dir.h>
#include <slink.h>

int fs_getattr(filesys * fs, inodesys * is, const char * path,
	struct stat * buf);

int fs_symlink(filesys * fs, inodesys * is, const char * path,
	const char * target);
int fs_readlink(filesys * fs, inodesys * is, const char * path, char * buf,
	size_t size);

int fs_unlink(filesys * fs, inodesys * is, const char * path);
int fs_link(filesys * fs, inodesys * is, const char * old, const char * new);
int fs_rename(filesys * fs, inodesys * is, const char * old, const char * new);

int fs_chmod(filesys * fs, inodesys * is, const char * path, mode_t mode);
int fs_chown(filesys * fs, inodesys * is, const char * path,
	uid_t uid, gid_t gid);
int fs_truncate(filesys * fs, inodesys * is, const char * path, off_t len);

int fs_create(filesys * fs, inodesys * is, const char * path, mode_t mode);
int fs_open(filesys * fs, inodesys * is, const char * path,
	struct fuse_file_info * info);
int fs_read(filesys * fs, inodesys * is, const char * path, char * buf,
	size_t size, off_t offset);
int fs_write(filesys * fs, inodesys * is, const char * path, const char * buf,
	size_t size, off_t offset);

int fs_opendir(filesys * fs, inodesys * is, const char * path,
	struct fuse_file_info * info);
int fs_readdir(filesys * fs, inodesys * is, const char * path, void * buf,
	fuse_fill_dir_t filler);
int fs_mkdir(filesys * fs, inodesys * is, const char * path, mode_t mode);
int fs_rmdir(filesys * fs, inodesys * is, const char * path);

int fs_init(char * filename, filesys ** fs, inodesys ** is, int * fsdes);
void fs_destroy(filesys * fs, inodesys * is, int fsdes);

#endif /* _FS_H */
