#include "inode.h"

/*** Utilites ***/

static int format(inodesys * sys)
{
    static inodesondisk zaroo; // Fresh and unused inodes

    if ( lseek(sys->fd, 0, SEEK_SET) == -1 ) {
	perror("inode: lseek");
	return 0;
    }

    if ( write(sys->fd, & zaroo, sizeof(inodesondisk)) != sizeof(inodesondisk) )
    {
	perror("inode: write");
	return 0;
    }

    return 1;
}

static int inode_in_use(inodesys * is, int n)
{
    int arraypos = n / CHAR_BIT;
    int bitpos = n % CHAR_BIT;
    return is->inodes->used[arraypos] & (1 << bitpos);
}

/*** Implementation ***/

inodesys * inode_init(int fd)
{
    struct stat st;
    inodesys * sys = NULL;

    sys = malloc(sizeof(inodesys));
    if ( sys == NULL ) {
	perror("inode: malloc");
	return NULL;
    }

    sys->inodes = mmap(NULL, sizeof(inodesondisk), PROT_READ | PROT_WRITE,
	    MAP_SHARED, fd, 0);
    if ( sys->inodes == NULL ) {
	free(sys);
	perror("inode: mmap");
    }

    sys->fd = fd;

    if ( fstat(fd, & st) == -1 ) {
	perror("inode: fstat");
	inode_destroy(sys);
	return NULL;
    }

    if ( st.st_size < sizeof(inodesondisk) )
	format(sys);
    
    return sys;
}

void inode_destroy(inodesys * sys)
{
    if ( munmap(sys->inodes, sizeof(inodesondisk) )
	    == -1 )
	perror("munmap");
    free(sys);
}

void inode_free(inodesys * sys, int n) {
    int arraypos = n / INODES;
    int bitpos = n % INODES;

    sys->inodes->used[arraypos] &= ~(1<<bitpos);
}

int inode_alloc(inodesys * sys)
{
    int i, j;
    inode * ino;
    static inode zaroO;

    // Look for free bits in each byte of the used array
    for ( i = 0 ; i < INODES / CHAR_BIT; i++ )
	// Look for a free bit in each bit of this byte
	if ( sys->inodes->used[i] != 255 )
	    for ( j = 0 ; j < CHAR_BIT ; j++ )
		// If the bit is not in use, use it and return it
		if ( ! (sys->inodes->used[i] & ( 1 << j ) ) ) {
		    sys->inodes->used[i] |= (1 << j);
		    i = i * CHAR_BIT + j;
		    // Fresh new inode
		    ino = inode_get(sys, i);
		    memcpy(ino, & zaroO, sizeof(inode));
		    return i;
		}

    return -1;
}

inode * inode_get(inodesys * sys, int n)
{
    if ( ! inode_in_use(sys, n) )
	return NULL;

    return sys->inodes->inodes + n;
}
