#ifndef _INODE_H
#define _INODE_H

/*
 * I-Node system to store "files" (it doesn't store file name)
 */

#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdio.h>

#define INODES 256

/*
 * This is one i-node
 */
typedef struct {
    mode_t mode;
    nlink_t nlink;
    uid_t uid;
    gid_t gid;
    size_t size;
    time_t access;
    time_t modification;
    time_t changestate;
    int address; // address of the first file block
} inode;

typedef struct {
    unsigned char used[INODES / CHAR_BIT]; // bit array to store used inodes
    inode inodes[INODES]; // mapped to the given file
} inodesondisk;

/*
 * This is the inode system
 */
typedef struct {
    int fd; // file used to store the inode system
    inodesondisk * inodes;
} inodesys;

/*
 * To init/destroy the inode system
 */
inodesys * inode_init(int fd);
void inode_destroy(inodesys * sys);

/*
 * To get free inodes and free them again
 *
 * Parameter sys must be created by a call to inode_init.
 * Parameter n is given by a precedent call to inode_alloc.
 */
int inode_alloc(inodesys * sys);
void inode_free(inodesys * sys, int n);

/*
 * To set/get inode data
 *
 * Parameter n is given by an inode_alloc call.
 * Parameter data must be memory allocated.
 */
inode * inode_get(inodesys * sys, int n);

#endif /* _INODE_H */
