#include <slink.h>

int slink_create(filesys * fs, inodesys * is, const char * path, size_t len)
{
    int inodenb;
    inode * ino;
    file * f;

    inodenb = inode_alloc(is);
    if ( inodenb < 0 )
	return -1;

    if ( (f = file_alloc(fs)) == NULL ) {
	inode_free(is, inodenb);
	return -1;
    }

    if ( ! file_set(fs, f, 0, len, path) ) {
	inode_free(is, inodenb);
	file_free(fs, f);
	return -1;
    }

    ino = inode_get(is, inodenb);
    if ( ino == NULL ) 
	return -1;

    ino->size = f->size;
    ino->address = f->faddrblck;

    free(f);

    return inodenb;
}

int slink_read(filesys * fs, file * f, char * buf, size_t size)
{
    return file_get(fs, f, 0, size, buf) ;
}
